/**
 * <h1>CallDeclaredGenerator</h1>
 *
 * <p>Generate code for a call to a declared routine.</p>
 *
 * <p>Copyright (c) 2018 by Ronald Mak</p>
 * <p>For instructional purposes only.  No warranties.</p>
 */
#ifndef CALLDECLAREDGENERATOR_H_
#define CALLDECLAREDGENERATOR_H_

#include "CallGenerator.h"
#include "ExpressionGenerator.h"
#include "../CodeGenerator.h"
#include "../../../intermediate/ICodeNode.h"

namespace wci { namespace backend { namespace compiler { namespace generators {

using namespace std;
using namespace wci::intermediate;
using namespace wci::backend::compiler;

class CallDeclaredGenerator : public CallGenerator
{
public:
    /**
     * Constructor.
     * @param the parent executor.
     */
    CallDeclaredGenerator(CodeGenerator *parent);

    /**
     * Generate code to call to a declared procedure or function.
     * @param node the CALL node.
     */
    void generate(ICodeNode *node);

private:
    /**
     * Generate code for the actual parameters of a call.
     * @param callNode the CALL parse tree node.
     */
    void generate_actual_parms(ICodeNode *call_node);

    /**
     * Wrap an actual parameter to pass it by reference.
     * in a procedure or function call.
     * @param actual_node the parse tree node of the actual parameter.
     * @param formal_typespec the type specification of the formal parameter
     * @param wrap_slot the slot number of the wrapper
     * @param expr_generator the expression code generator.
     */
    void generate_wrap(ICodeNode *actual_node, TypeSpec *formal_typespec,
                       int wrap_slot, ExpressionGenerator& expr_generator);

    /**
     * Clone an actual parameter value to be passed by value
     * in a procedure or function call.
     * @param formal_typespec the type specification of the formal parameter
     */
    void clone_actual_parameter(TypeSpec *formal_typespec);

    /**
     * Generate code to make the call.
     * @param call_node the CALL parse tree node.
     */
    void generate_call(ICodeNode *call_node);

    /**
     * Generate code for the call epilogue.
     * @param call_node the CALL parse tree node
     */
    void generate_call_epilogue(ICodeNode *call_node);

    /**
     * Generate the code to unwrap an actual parameter value.
     * @param actual_id the symbol table entry of the actual identifier.
     * @param formal_typespec the type specification of the formal parameter
     * @param program_name the name of the program.
     */
    void generate_unwrap(SymTabEntry *actual_id, TypeSpec *formal_typespec,
                         string program_name);
};

}}}}  // namespace wci::backend::compiler::generators

#endif /* CALLDECLAREDGENERATOR_H_ */
