/**
 * <h1>CallStandardGenerator</h1>
 *
 * <p>Generate code for a call to a standard routine.</p>
 *
 * <p>Copyright (c) 2018 by Ronald Mak</p>
 * <p>For instructional purposes only.  No warranties.</p>
 */
#ifndef CALLSTANDARDGENERATOR_H_
#define CALLSTANDARDGENERATOR_H_

#include "CallGenerator.h"
#include "ExpressionGenerator.h"
#include "../CodeGenerator.h"
#include "../../../intermediate/ICodeNode.h"

namespace wci { namespace backend { namespace compiler { namespace generators {

using namespace std;
using namespace wci::intermediate;
using namespace wci::backend::compiler;

class CallStandardGenerator : public CallGenerator
{
public:
    /**
     * Constructor.
     * @param the parent executor.
     */
    CallStandardGenerator(CodeGenerator *parent) : CallGenerator(parent) {}

    /**
     * Generate code to call to a standard procedure or function.
     * @param node the CALL node.
     * @return the function value, or null for a procedure call.
     */
    void generate(ICodeNode *node);

private:
    /**
     * Generate code for a call to read or readln.
     * @param call_nNode the CALL node.
     * @param routine_code the routine code.
     */
    void generate_read_readln(ICodeNode *call_node,
                              RoutineCode routine_code);

    /**
     * Generate code for a call to eof or eoln.
     * @param call_node the CALL node.
     * @param routine_code the routine code.
     */
    void generate_eof_eoln(ICodeNode *call_node, RoutineCode routine_code);

    /**
     * Generate code for a call to write or writeln.
     * @param call_node the CALL node.
     * @param routine_code the routine code.
     */
    void generate_write_writeln(ICodeNode *call_node,
                                RoutineCode routine_code);

    /**
     * Generate code for a call to abs or sqr.
     * @param routine_code the routine code.
     * @param actual_node the actual parameter node.
     */
    void generate_abs_sqr(RoutineCode routine_code, ICodeNode *actual_node);

    /**
     * Generate code for a call to arctan, cos, exp, ln, sin, or sqrt.
     * @param routine_code the routine code.
     * @param actual_node the actual parameter node.
     */
    void generate_arctan_cos_exp_ln_sin_qqrt(RoutineCode routine_code,
                                             ICodeNode *actual_node);

    /**
     * Generate code for a call to pred or succ.
     * @param routine_code the routine code.
     * @param actual_node the actual parameter node.
     */
    void generate_pred_succ(RoutineCode routine_code,
                            ICodeNode *actual_node);

    /**
     * Generate code for a call to chr.
     * @param actual_node the actual parameter node.
     */
    void generate_chr(ICodeNode *actual_node);

    /**
     * Generate code for a call to odd.
     * @param actual_node the actual parameter node.
     */
    void generate_odd(ICodeNode *actual_node);

    /**
     * Generate code for a call to ord.
     * @param actual_node the actual parameter node.
     */
    void generate_ord(ICodeNode *actual_node);

    /**
     * Generate code for a call to round or trunc.
     * @param routine_code the routine code.
     * @param actual_node the actual parameter node.
     */
    void generate_round_trunc(RoutineCode routine_code,
                              ICodeNode *actual_node);
};

}}}}  // namespace wci::backend::compiler::generators

#endif /* CALLSTANDARDGENERATOR_H_ */
