/**
 * <h1>IfGenerator</h1>
 *
 * <p>Generate code for an IF statement.</p>
 *
 * <p>Copyright (c) 2018 by Ronald Mak</p>
 * <p>For instructional purposes only.  No warranties.</p>
 */

#include <string>
#include <vector>
#include "IfGenerator.h"
#include "StatementGenerator.h"
#include "ExpressionGenerator.h"
#include "../CodeGenerator.h"
#include "../../../intermediate/ICodeNode.h"

namespace wci { namespace backend { namespace compiler { namespace generators {

using namespace std;
using namespace wci::intermediate;
using namespace wci::backend::compiler;

IfGenerator::IfGenerator(CodeGenerator *parent)
    : StatementGenerator(parent)
{
}

void IfGenerator::generate(ICodeNode *node) throw (string)
{
    vector<ICodeNode *> *children = node->get_children();
    ICodeNode *expression_node = (*children)[0];
    ICodeNode *then_node = (*children)[1];
    ICodeNode *else_node = children->size() > 2 ? (*children)[2] : nullptr;
    Label *next_label = Label::new_label();
    ExpressionGenerator expr_generator(this);
    StatementGenerator  stmt_generator(this);

    // Generate code for the boolean expression.
    expr_generator.generate(expression_node);

    // Generate code for a THEN statement only.
    if (else_node == nullptr)
    {
        emit(IFEQ, next_label);
        local_stack->decrease(1);

        stmt_generator.generate(then_node);
    }

    // Generate code for a THEN statement and an ELSE statement.
    else
    {
        Label *false_label = Label::new_label();

        emit(IFEQ, false_label);
        local_stack->decrease(1);

        stmt_generator.generate(then_node);
        emit(GOTO, next_label);

        emit_label(false_label);
        stmt_generator.generate(else_node);
    }

    emit_label(next_label);
}

}}}}  // namespace wci::backend::compiler::generators
