/**
 * <h1>IfGenerator</h1>
 *
 * <p>Generate code for a looping statement.</p>
 *
 * <p>Copyright (c) 2018 by Ronald Mak</p>
 * <p>For instructional purposes only.  No warranties.</p>
 */
#include <string>
#include "LoopGenerator.h"
#include "StatementGenerator.h"
#include "ExpressionGenerator.h"
#include "../CodeGenerator.h"
#include "../../../intermediate/ICodeNode.h"

namespace wci { namespace backend { namespace compiler { namespace generators {

using namespace std;
using namespace wci::intermediate;
using namespace wci::backend::compiler;

LoopGenerator::LoopGenerator(CodeGenerator *parent)
    : StatementGenerator(parent)
{
}

void LoopGenerator::generate(ICodeNode *node) throw (string)
{
    vector<ICodeNode *> *loop_children = node->get_children();
    Label *loop_label = Label::new_label();
    Label *next_label = Label::new_label();
    StatementGenerator  stmt_generator(this);
    ExpressionGenerator expr_generator(this);

    emit_label(loop_label);

    // Generate code for the children of the LOOP node.
    for (ICodeNode *child : *loop_children)
    {
        ICodeNodeType child_type = child->get_type();

        // TEST node: Generate code to test the boolean expression.
        if (child_type == (ICodeNodeType) NT_TEST)
        {
            ICodeNode *expression_node = (*child->get_children())[0];

            expr_generator.generate(expression_node);
            emit(IFNE, next_label);

            local_stack->decrease(1);
        }

        // Statement node: Generate code for the statement.
        else
        {
            stmt_generator.generate(child);
        }
    }

    emit(GOTO, loop_label);
    emit_label(next_label);
}

}}}}  // namespace wci::backend::compiler::generators
