/**
 * <h1>PascalErrorToken</h1>
 *
 * <p>Pascal error token.</p>
 *
 * <p>Copyright (c) 2017 by Ronald Mak</p>
 * <p>For instructional purposes only.  No warranties.</p>
 */
#ifndef WCI_FRONTEND_PASCAL_TOKENS_PASCALERRORTOKEN_H_
#define WCI_FRONTEND_PASCAL_TOKENS_PASCALERRORTOKEN_H_

#include <string>
#include "../PascalError.h"
#include "../PascalToken.h"

namespace wci { namespace frontend { namespace pascal { namespace tokens {

using namespace std;
using namespace wci::frontend::pascal;

class PascalErrorToken : public PascalToken
{
public:
    /**
     * Constructor.
     * @param source the source from where to fetch subsequent characters.
     * @param errorCode the error code.
     * @param tokenText the text of the erroneous token.
     * @throw a string message if an error occurred.
     */
    PascalErrorToken(Source *source, PascalErrorCode error_code,
                     string token_text)
        throw (string)
        : PascalToken(source)
    {
        type = (TokenType) PT_ERROR;
        text = token_text;
        value = (int) error_code;
    }

protected:
    /**
     * Do nothing.  Do not consume any source characters.
     * Override of wci::frontend::Token.
     * @throw a string message if an error occurred.
     */
    void extract() throw (string)
    {
        // do nothing
    }
};

}}}}  // namespace wci::frontend::pascal::tokens

#endif /* WCI_FRONTEND_PASCAL_TOKENS_PASCALERRORTOKEN_H_ */
